/*
This software is available under NYSL(Nirunari Yakunari Sukinishiro License).
*/

/* ステッピングモータドライバ(両輪) */

#include <rxduino.h>

/** イネーブルのピン番号 */
static int pinEnable;
/** 左輪方向のピン番号 */
static int pinLeftDir;
/** 左輪クロックのピン番号 */
static int pinLeftClk;
/** 右輪方向のピン番号 */
static int pinRightDir;
/** 右輪クロックのピン番号 */
static int pinRightClk;
/** クロック生成用のカウンタ[ms] */
static int msCnt;
/** クロックの半周期[ms] (ゼロのとき停止) */
static int msPeriod;

/**
 * 1ms周期タイマコールバック
 */
void steppers_callback(void)
{
    static int toggle = 0;
    
    /* 周期が設定されてるか？ (停止中はゼロ) */
    if(msPeriod != 0)
    {
        msCnt++;
        /* 周期ごとにクロック信号をトグル */
        if(msCnt >= msPeriod)
        {
            msCnt=0;
            toggle = 1- toggle; // 0<->1 toggle
            digitalWrite(pinLeftClk,  toggle);  /* 左輪 */
            digitalWrite(pinRightClk, toggle);  /* 右輪 */
        }
    }
}

/**
 * 初期化する
 *
 * @param en イネーブルのピン番号
 * @param l_clk 左輪のクロックのピン番号
 * @param l_dir 左輪の方向のピン番号
 * @param r_clk 右輪のクロックのピン番号
 * @param r_dir 右輪の方向のピン番号
 */
void steppers_init(int en, int l_clk, int l_dir, int r_clk, int r_dir)
{
    pinEnable   = en;
    pinLeftDir  = l_dir;
    pinLeftClk  = l_clk;
    pinRightDir = r_dir;
    pinRightClk = r_clk;
    
    /* GPIOの初期値設定 */
    digitalWrite(pinEnable,   0); /* ディセーブル */
    digitalWrite(pinLeftDir,  0);
    digitalWrite(pinLeftClk,  0);
    digitalWrite(pinRightDir, 0);
    digitalWrite(pinRightClk, 0);
    
    /* GPIOの方向設定(出力) */
    pinMode(pinEnable,  OUTPUT);
    pinMode(pinLeftDir, OUTPUT);
    pinMode(pinLeftClk, OUTPUT);
    pinMode(pinRightDir,OUTPUT);
    pinMode(pinRightClk,OUTPUT);
    
    /* 初期状態は停止 */
    msPeriod = 0;
    
    /* 1ms周期タイマコールバックの登録 */
    timer_regist_userfunc( steppers_callback );
}

/**
 * 前進する
 *
 * @param speed 速度(クロックの半周期[ms], 小さいほど速い)
 */
void steppers_forward(int speed)
{
    digitalWrite(pinLeftDir,  1);
    digitalWrite(pinRightDir, 0);
    msCnt = 0;
    msPeriod = speed;
}

/**
 * 後退する
 *
 * @param speed 速度(クロックの半周期[ms], 小さいほど速い)
 */
void steppers_backward(int speed)
{
    digitalWrite(pinLeftDir,  0);
    digitalWrite(pinRightDir, 1);
    msCnt = 0;
    msPeriod = speed;
}

/**
 * 右回転する
 *
 * @param speed 速度(クロックの半周期[ms], 小さいほど速い)
 */
void steppers_turnRight(int speed)
{
    digitalWrite(pinLeftDir,  1);
    digitalWrite(pinRightDir, 1);
    msCnt = 0;
    msPeriod = speed;
}

/**
 * 左回転する
 *
 * @param speed 速度(クロックの半周期[ms], 小さいほど速い)
 */
void steppers_turnLeft(int speed)
{
    digitalWrite(pinLeftDir,  0);
    digitalWrite(pinRightDir, 0);
    msCnt = 0;
    msPeriod = speed;
}

/**
 * 停止する
 */
void steppers_stop(void)
{
    digitalWrite(pinLeftDir,  0);
    digitalWrite(pinRightDir, 1);
    msCnt = 0;
    msPeriod = 0;
}

/**
 * イネーブルする
 */
void steppers_enable(void)
{
    digitalWrite(pinEnable,   1);
}

/**
 * ディセーブルする
 */
void steppers_disable(void)
{
    digitalWrite(pinEnable,   0);
}


